/*
 * Awake File: Easy file upload & download through HTTP with Java
 * Awake SQL: Remote JDBC access through HTTP.                                    
 * Copyright (C) 2012, Kawan Softwares S.A.S.
 * (http://www.awakeframework.org). All rights reserved.                                
 *                                                                               
 * Awake File/SQL is free software; you can redistribute it and/or                 
 * modify it under the terms of the GNU Lesser General Public                    
 * License as published by the Free Software Foundation; either                  
 * version 2.1 of the License, or (at your option) any later version.            
 *                                                                               
 * Awake File/SQL is distributed in the hope that it will be useful,               
 * but WITHOUT ANY WARRANTY; without even the implied warranty of                
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU             
 * Lesser General Public License for more details.                               
 *                                                                               
 * You should have received a copy of the GNU Lesser General Public              
 * License along with this library; if not, write to the Free Software           
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  
 * 02110-1301  USA
 *
 * Any modifications to this file must keep this entire header
 * intact.
 */

//HttpConfigurationUtil.java
//Copyright (c) Kawan Softwares S.A.S, 2012
//
//Last Updates: 
// 7 janv. 2010 12:56:20 Nicolas de Pomereu
// 24/03/10 16:50 NDP - Force path creation for filename if it not exists
// 04/06/11 11:50 NDP : HttpConfigurationUtil: rename username to username
// 07/12/11 18:305 NDP : HttpConfigurationUtil: clean javadoc
// 10/01/12 18:20 NDP : HttpConfigurationUtil: adapt code because 
//			AwakeFileConfigurator.getServerRoot()returns now a File
// 10/01/12 18:20 NDP : HttpConfigurationUtil: no more "users" dir
// 11/01/12 13:45 NDP : HttpConfigurationUtil: clean comments & modify / to  
//		        File.separator if no server root

package org.awakefw.file.api.server.fileaction;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.IOException;

import org.apache.commons.lang3.SystemUtils;
import org.awakefw.file.api.server.AwakeFileConfigurator;
import org.awakefw.file.servlet.ServerUserException;
import org.awakefw.file.util.Tag;

/**
 * 
 * Utility class to use with <code>AwakeFileActionManager</code> implementation
 * 
 * @author Nicolas de Pomereu
 * @since 1.0
 * 
 */
public class HttpConfigurationUtil {

    /**
     * Protected constructor
     */
    protected HttpConfigurationUtil() {

    }

    /**
     * Adds the root path to the beginning of path for a remote file.&nbsp;
     * <p>
     * If the Awake server runs on Windows: the method will convert the "/" file
     * separator to "\" if necessary.
     * 
     * @param awakeFileConfigurator
     *            the user http configuration
     * @param username
     *            the client username
     * @param filename
     *            the filename path
     * 
     * @return the new filename path, prefixed with the root path
     */
    public static String addRootPath(
	    AwakeFileConfigurator awakeFileConfigurator, String username,
	    String filename) throws IOException {
	if (filename == null) {
	    throw new IllegalArgumentException(Tag.AWAKE_PRODUCT_FAIL
		    + "filename can not be null!");
	}

	File serverRootFile = null;
	
	try {
	    serverRootFile = awakeFileConfigurator.getServerRoot();
	} catch (Exception e) {
	    ServerUserException.throwUserIOException(awakeFileConfigurator, e,
		    "getServerRoot");
	}
	
	if (awakeFileConfigurator == null
		|| serverRootFile== null) {

	    if (SystemUtils.IS_OS_WINDOWS) {
		// Replace all "/" with "\" for Windows
		filename = filename.replace("/", File.separator);
	    }

	    return filename;
	}


	if (!serverRootFile.exists() || !serverRootFile.isDirectory()) {
	    serverRootFile.mkdirs();
	}

	if (!serverRootFile.isDirectory()) {
	    throw new FileNotFoundException(
		    Tag.AWAKE_PRODUCT_FAIL
			    + "Server root directory does not exists and can not be created: "
			    + serverRootFile.toString());
	}
	
	boolean doUseOneRootPerUsername = false;
	
	try {
	    doUseOneRootPerUsername = awakeFileConfigurator.useOneRootPerUsername();
	} catch (Exception e) {
	    ServerUserException.throwUserIOException(awakeFileConfigurator, e,
		    "useOneRootPerUsername");
	}
	
	if (doUseOneRootPerUsername) {
	    // Add username to the root directory
	    serverRootFile = new File(serverRootFile.toString() + File.separator + username); 
	}

	if (SystemUtils.IS_OS_WINDOWS) {
	    if (filename.contains(":\\") && filename.length() >= 3) {
		filename = filename.substring(3);
	    }

	    // Replace all "/" with "\" for Windows
	    filename = filename.replace("/", File.separator);

	} else {
	    if (filename.startsWith("/") && filename.length() >= 1) {
		filename = filename.substring(1);
	    }
	}

	filename = serverRootFile.toString() + File.separator + filename;

	// Force path creation for filename if it not exists
	File file = new File(filename);
	if (file.getParent() != null) {
	    file.getParentFile().mkdirs();
	}

	return filename;
    }

}
